Attribute VB_Name = "mdlCrypt"
Option Explicit

Private Declare Function ASM_cdLong _
    Lib "user32" Alias "CallWindowProcA" _
   (ByRef asm As Long, _
    ByVal PA1 As Long, _
    ByVal PA2 As Long, _
    ByVal PA3 As Long, _
    ByVal PA4 As Long) As Long


' Die eigentlichen Kernfunktionen:
' Zum Signieren und Verschlsseln werden zwar die gleichen Funktionen verwendet, aber zum besseren Auseinanderhalten zweimal implementiert^^

Public Function Encrypt(ByVal Message As Long, ByVal Party As Party) As Long
   Encrypt = ModExp(Message, Party.e, Party.N)
End Function

Public Function TestSignature(ByVal Signature As Long, ByVal Message As Long, ByVal Party As Party) As Boolean
   TestSignature = (ModExp(Signature, Party.e, Party.N) = Message)
End Function

Public Function GetSignaturePlaintext(ByVal Signature As Long, ByVal Party As Party) As Long
   GetSignaturePlaintext = ModExp(Signature, Party.e, Party.N)
End Function
   
   
   
' Implementierung von bentigten mathematischen Funktionen:

' Berechnung einer Zufallszahl innerhalb vorgegebener Grenzen
Public Function Random(ByVal Min As Long, ByVal Max As Long) As Long
   Random = Int(Rnd * (Max - Min)) + Min
End Function

' Hochzhlen bis zur nchsten Primzahl
Public Function NextPrime(Number As Long) As Long
   Dim i As Long
   
   i = Number + IIf((Number And 1) = 1, 0, 1)
   
   Do While Not IsPrime(i)
      i = i + 2
   Loop
    
   NextPrime = i
End Function

' Ist eine Zahl eine Primzahl?
Private Function IsPrime(ByVal Number As Long) As Boolean
   Dim i As Long
    
   For i = 2 To Sqr(Number - 1)
      If Number Mod i = 0 Then
         IsPrime = False
         Exit Function
      End If
   Next i
    
   IsPrime = True
End Function

' Modulare Exponentation

' ' VB6-Version ist leider zu langsam:
'Public Function ModExp(Basis As Long, ByVal Exponent As Long, Modul As Long) As Long
'    ModExp = 1
'
'    While Exponent > 0
'      ModExp = ModExp * Basis Mod Modul
'      Exponent = Exponent - 1
'    Wend
'End Function


' ASM-Beschleunigte Funktion
' Hinweis: Dieser Code stammt von Udo Schmidt!
Public Function ModExp(Base As Long, Exp As Long, Module As Long) As Long
    Static asm(8) As Long

    If asm(0) = 0 Then
        asm(0) = &H748B5756:  asm(1) = &H4C8B0C24:  asm(2) = &H7C8B1024
        asm(3) = &H1B81424:   asm(4) = &HF7000000:  asm(5) = &H8BF7F7E6
        asm(6) = &HF77549C2:  asm(7) = &H10C25E5F:  asm(8) = &H0
    End If

    ModExp = ASM_cdLong(asm(0), Base, Exp, Module, 0)
End Function

' Finden der kleinstmglichen teilerfremden Zahl
Public Function CoPrime(ByVal CoPrimeTo As Long) As Long
   Dim i As Long
   i = 2
   
   Do
      Do
         i = i + 1
         If i > CoPrimeTo Then Exit Function
      Loop While ggT(i, CoPrimeTo) <> 1
   Loop While EuklidEx(i, CoPrimeTo) < 0
   
   CoPrime = i
End Function

' Berechnung des grten gemeinsamen Teilers zweier Zahlen
Private Function ggT(ByVal a As Long, ByVal b As Long) As Long
  Dim Tmp As Long

  Tmp = a Mod b
  
  If Tmp = 0 Then
    ggT = b
    Exit Function
  Else
    a = b
    b = Tmp
  End If
  
  While a Mod b <> 0
    Tmp = a Mod b
    a = b
    b = Tmp
  Wend
  
  ggT = Tmp
End Function

' Berechnung des multiplikativ Inversen durch den erweiterten euklidischen Algorithmus
Function EuklidEx(ByVal a As Long, ByVal b As Long) As Long
   Dim x1 As Long, xtmp As Long, y1 As Long, ytmp As Long, r As Long, t As Long, q As Long, x0 As Long, y0 As Long
   
   x0 = 1
   y1 = 1
   t = 1
   
   Do While b <> 0
      r = a Mod b
      q = a \ b
      a = b
      b = r
      xtmp = x1
      ytmp = y1
      x1 = q * x1 + x0
      y1 = q * y1 + y0
      x0 = xtmp
      y0 = ytmp
      t = -t
  Loop
  
  EuklidEx = x0 * t
   
End Function


